#ifndef IU9_CA_WEB_CHAT_LIB_BACKEND_LOGIC_SERVER_DATA_INTERACT_H
#define IU9_CA_WEB_CHAT_LIB_BACKEND_LOGIC_SERVER_DATA_INTERACT_H

/* This folder covers all code that helps to interact with database and templater,
*  or dictates the logic of how this web chat functions */

#include <memory>
#include <new_york_transit_line/templater.h>
#include "../sqlite3_wrapper.h"
#include "../login_cookie.h"
#include <engine_engine_number_9/http_structures/client_request.h>
#include <engine_engine_number_9/http_structures/response_gen.h>
#include <jsonincpp/string_representation.h>

namespace iu9cawebchat {
    constexpr int64_t user_chat_role_admin = 1;
    constexpr int64_t user_chat_role_regular = 2;
    constexpr int64_t user_chat_role_read_only = 3;
    constexpr int64_t user_chat_role_deleted = 4;

    const char* stringify_user_chat_role(int64_t role);

    struct WorkerGuestData {
        /* Because templaters use libjsonincpp, they can't be READ by two thread simultaneously */
        std::unique_ptr<nytl::Templater> templater;
        std::unique_ptr<SqliteConnection> db;
        int debug_trans_op; // todo: delete when debug is over
    };

    int64_t find_user_by_credentials (SqliteConnection& conn, const std::string& nickname, const std::string& password);
    std::string find_user_name (SqliteConnection& conn, int64_t uid);

    struct RowUser_Content {
        std::string nickname;
        std::string name;
    };

    struct RowChat_Content {
        std::string nickname;
        std::string name;
        int64_t lastMsgId;  // Negative if it does not exist
    };

    RowUser_Content lookup_user_content(SqliteConnection& conn, int64_t uid);
    RowChat_Content lookup_chat_content(SqliteConnection& conn, int64_t uid);


    void initial_extraction_of_all_the_useful_info_from_cookies(
        SqliteConnection& conn, const een9::ClientRequest& req,
        std::vector<std::pair<std::string, std::string>>& ret_cookies,
        std::vector<LoginCookie>& ret_login_cookies,
        json::JSON& ret_userinfo,
        int64_t& ret_logged_in_user
    );

    int64_t get_role_of_user_in_chat(SqliteConnection& conn, int64_t userId, int64_t chatId);

    std::string RTEE(const std::string& el_name,
        const json::JSON& config_presentation, WorkerGuestData& wgd,
        const json::JSON& userinfo);

    /*  ========================== PAGES ================================== */
    std::string when_page_login(WorkerGuestData& wgd, const json::JSON& config_presentation,
        const een9::ClientRequest& req, const std::vector<LoginCookie>& login_cookies, const json::JSON& userinfo);


    json::JSON internalapi_pollEvents(SqliteConnection& conn, int64_t uid, const json::JSON& Sent);

    std::string when_internalapi_pollevents(WorkerGuestData& wgd,
            const een9::ClientRequest& req, int64_t uid);


    json::JSON internalapi_getChatList(SqliteConnection& conn, int64_t uid);

    std::string when_internalapi_getchatlist(WorkerGuestData& wgd,
                const een9::ClientRequest& req, int64_t uid);
}

#endif
